"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const consts_1 = require("../consts");
function deserialize(json, classType) {
    return transform(JSON.parse(json), classType);
}
exports.deserialize = deserialize;
// -- Private --
function transformArray(arr, classType) {
    return arr.map((elm) => (Array.isArray(elm) ? transformArray(elm, classType) : transform(elm, classType)));
}
function transform(obj, classType) {
    const instance = new classType();
    const typeMap = Reflect.getMetadata(consts_1.TypeSymbol, instance) || {};
    Object.keys(obj).forEach((key) => {
        instance[key] = obj[key];
        if (!typeMap.hasOwnProperty(key)) {
            return;
        }
        const type = typeMap[key];
        if (Array.isArray(type) && Array.isArray(obj[key])) {
            if (type.length !== 1) {
                throw new Error('`@Type` can only be defined with a single value, or an array with a single value. for ex: `@Type(User)` or `@Type([User])`');
            }
            instance[key] = transformArray(obj[key], type[0]);
        }
        else if (type === Date) {
            instance[key] = new Date(obj[key]);
        }
        else {
            instance[key] = transform(obj[key], type);
        }
    });
    return instance;
}
//# sourceMappingURL=Deserializer.js.map