"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const consts_1 = require("../consts");
const helpers_1 = require("../helpers");
function serialize(obj, groups, version) {
    if (Array.isArray(obj)) {
        return JSON.stringify(transformArray(obj, groups, version));
    }
    return JSON.stringify(transform(obj, groups, version));
}
exports.serialize = serialize;
// -- Private --
function transformArray(arr, groups, version) {
    return arr.map((elm) => (Array.isArray(elm) ? transformArray(elm, groups, version) : transform(elm, groups, version)));
}
function transform(obj, groups, version) {
    const excludeMap = Reflect.getMetadata(consts_1.ExcludeSymbol, obj) || {};
    const exposeMap = Reflect.getMetadata(consts_1.ExposeSymbol, obj) || {};
    const groupsMap = Reflect.getMetadata(consts_1.GroupsSymbol, obj) || {};
    const beforeMap = Reflect.getMetadata(consts_1.BeforeSymbol, obj) || {};
    const afterMap = Reflect.getMetadata(consts_1.AfterSymbol, obj) || {};
    const nameMap = Reflect.getMetadata(consts_1.NameSymbol, obj) || {};
    const strategy = Reflect.getMetadata(consts_1.StrategySymbol, obj.constructor);
    return Object.getOwnPropertyNames(obj).reduce((json, key) => {
        const name = nameMap[key] || key;
        if (shouldAdd(obj, excludeMap, exposeMap, beforeMap, afterMap, groupsMap, strategy, key, groups, version)) {
            if (Array.isArray(obj[key])) {
                json[name] = transformArray(obj[key], groups, version);
            }
            else if (helpers_1.isObject(obj[key])) {
                json[name] = transform(obj[key], groups, version);
            }
            else {
                json[name] = obj[key];
            }
        }
        return json;
    }, {});
}
function shouldAdd(obj, excludeMap, exposeMap, beforeMap, afterMap, groupsMap, strategy, key, groups, version) {
    const propGroups = groupsMap && groupsMap[key] ? groupsMap[key] : [];
    if (strategy === consts_1.ExclusionPolicy.ALL && (!exposeMap.hasOwnProperty(key) || !exposeMap[key].call(null, obj, key))) {
        return false;
    }
    if (excludeMap.hasOwnProperty(key) && excludeMap[key].call(null, obj, key)) {
        return false;
    }
    if (version && beforeMap.hasOwnProperty(key)) {
        return helpers_1.versionCompare(beforeMap[key], version) === 1;
    }
    if (version && afterMap.hasOwnProperty(key)) {
        return helpers_1.versionCompare(afterMap[key], version) <= 0;
    }
    if (groups && groups.length) {
        return groups.some((group) => !!~propGroups.indexOf(group));
    }
    return true;
}
//# sourceMappingURL=Serializer.js.map